@@ -17,43 +17,39 @@ factorMatch = function (x,xlevels)
 
    if (!is.null(attr(x,"illegalLevelCounts"))) return(x = x)
    facts=intersect(names(x),names(xlevels))
-   if (length(facts)==0) return(x = x)
+   if (length(facts)==0)
+   {
+     attr(x,"illegalLevelCounts")= 0
+     return(x = x)
+   }
 
    nas=NULL
    miss=NULL
    for (varName in facts)
-   {  	
-      origLevs= unique(I(as.character(x[,varName])))
-      lt=xlevels[[varName]]
-      fn=lt[suppressWarnings(as.numeric(origLevs))]
-      if (any(is.na(fn))) fn[is.na(fn)] = origLevs[is.na(fn)]
-      x[,varName]=as.factor(x[,varName])
-      attr(x[,varName],"levels")=fn
-      new = match(x[,varName],xlevels[[varName]])
-      nas=is.na(new)
-      if (any(nas))
+   {
+      fx = factor(x[,varName],xlevels[[varName]])
+      if (any(is.na(fx)))
       {
-         mtb=table(as.character(x[nas,varName]))
-         if(is.null(miss)) 
-         {
-           miss=list(mtb)
-           names(miss)=varName
-          } 
-          else 
-          {
-             miss=c(miss,list(mtb))
-             names(miss)[length(miss)]=varName
-          }
-      }
-      class(new)="factor"
-      attr(new,"levels")=xlevels[[varName]]
-      x[,varName]=new
+        lx = table(factor(x[,varName]))
+        mtb = lx[setdiff(names(lx),xlevels[[varName]])]
+        if(is.null(miss))
+        {
+          miss=list(mtb)
+          names(miss)=varName
+        }
+        else
+        {
+          miss=c(miss,list(mtb))
+          names(miss)[length(miss)]=varName
+        }
+     }
+     x[,varName]=fx
    }
    if (is.null(miss))
-   { 
-      attr(x,"illegalLevelCounts")= 0 
+   {
+      attr(x,"illegalLevelCounts")= 0
    }
-   else 
+   else
    {
    	  x=na.omit(x)
       attr(x,"illegalLevelCounts")= miss
